<?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');
/*********************************************************************************
 * SugarCRM is a customer relationship management program developed by
 * SugarCRM, Inc. Copyright (C) 2004 - 2007 SugarCRM Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License version 3 as published by the
 * Free Software Foundation with the addition of the following permission added
 * to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED WORK
 * IN WHICH THE COPYRIGHT IS OWNED BY SUGARCRM, SUGARCRM DISCLAIMS THE WARRANTY
 * OF NON INFRINGEMENT OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, see http://www.gnu.org/licenses or write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301 USA.
 * 
 * You can contact SugarCRM, Inc. headquarters at 10050 North Wolfe Road,
 * SW2-130, Cupertino, CA 95014, USA. or at email address contact@sugarcrm.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU General Public License version 3,
 * these Appropriate Legal Notices must retain the display of the "Powered by
 * SugarCRM" logo. If the display of the logo is not reasonably feasible for
 * technical reasons, the Appropriate Legal Notices must display the words
 * "Powered by SugarCRM".
 ********************************************************************************/

/**
 * DetailViewMetaParser.php
 * This is a utility file that attempts to provide support for parsing pre 5.0 SugarCRM 
 * DetailView.html files and produce a best guess detailviewdefs.php file equivalent.
 * 
 * @author Collin Lee
 */
require_once('include/SugarFields/Parsers/MetaParser.php');

/**
 * DetailViewMetaParser.php
 * This class is responsible for handling the parsing of DetailView.html files from
 * SugarCRM versions prior to 5.x.  It will make a best guess of translating the
 * HTML file to a metadata format.
 * 
 * @author Collin Lee
 */
class DetailViewMetaParser extends MetaParser {

function DetailViewMetaParser() {
   $this->mView = 'DetailView';
}


/**
 * parse
 * 
 * @param $filePath The file path of the HTML file to parse
 * @param $vardefs The module's vardefs
 * @param $moduleDir The module's directory
 * @param $merge boolean value indicating whether or not to merge the parsed contents
 * @param $masterCopy The file path of the mater copy of the metadata file to merge against
 * @return String format of metadata contents
 **/
function parse($filePath, $vardefs = array(), $moduleDir = '', $merge=false, $masterCopy=null) {
   
// Grab file contents
$contents = file_get_contents($filePath);
   
// Remove \n,\r characters to allow for better text parsing
$contents = $this->trimHTML($contents);
   
// Get all the tables
$tables = $this->getElementsByType("table", $contents);
   
// Skip the first one
$tables = array_slice($tables, 1);

$panels = array();
$tableCount = 0;
$metarow = array();

foreach($tables as $table) {   
               
   $tablerows = $this->getElementsByType("tr", $table);

   foreach($tablerows as $trow) {

       $metacolumns = array();  
   	   $columns = $this->getElementsByType("td", $trow);

   	   // We reverse the array since we want to first process the slot[0-9]+b tags
   	   $tablecolumns = array_reverse($columns, true);
   	   
       $def = null;
       $slot = 0;
	   foreach($tablecolumns as $tcols) {

	   	  $field = strtolower($this->getElementValue("span", $tcols));
	   	  if(empty($field) || $field == '') {
	   	  	 $field = $tcols;
	   	  }
	   	  //If there are HTML tags, do not use as field, but only get contents between {} tags
	   	  if(preg_match('/^[<][^>]*?[>]([\{][^\}].*?[\}])<\/.*?/s', $field, $m)) { 
	   	  	 $field = $m[1];
	   	  } //if

	   	  $sugarAttrValue = $this->getTagAttribute("sugar", $tcols, "'slot[0-9]+b$'");
	      $sugarAttrValue = is_array($sugarAttrValue) && count($sugarAttrValue) != 0 ? $sugarAttrValue : ($slot % 2 == 0) ? true : false;
	      $slot++;

          //Grab all values enclosed within {} tags
          preg_match_all('/[\{]([^\}].*?)[\}]/s', $field, $matches, PREG_SET_ORDER);
	   	  
	   	  if($sugarAttrValue) {         
	   	  	 if($matches) {	 
	   	  	 	 //Check if there are more than one tag with {} enclosed, treat as custom code
		   	  	 if(count($matches) > 1) {
		   	  	 	 // Keep $mod_strings and $app_strings as uppercased variables
		   	  	 	 
		   	  	 	 foreach($matches as $tag[1]) {
		   	  	 	    if(preg_match("/^(mod[\.]|app[\.]).*?/s", $tag[1][1])) {
		   	  	 	       $field = str_replace($tag[1][1], '$'.strtoupper($tag[1][1]), $field);      
		   	  	 	    } else {
		   	  	 	       if(!empty($vardefs[$tag[1][1]])) {
		   	  	 	          $field = str_replace($tag[1][1], '$fields.'.$tag[1][1].'.value', $field);
		   	  	 	       } else {
		   	  	 	       	  $phpName = $this->findAssignedVariableName($tag[1][1], $filePath);
		   	  	 	       	  $field = str_replace($tag[1][1], '$fields.'.$phpName.'.value', $field);
		   	  	 	       } //if-else
		   	  	 	    }
		   	  	 	 }
		   	  	 	 
		   	  	 	 $def = array();
		   	  	 	 $def['name'] = isset($phpName) ? $phpName : $tag[1][1];
		   	  	 	 $def['customCode'] = $field; 
		   	  	 } else {

			   	  	 foreach($matches as $tag[1]) {
			   	  	 	     if(!empty($tag[1][0])) {
			   	  	 	        if(!empty($vardefs[$tag[1][1]])) {
			   	  	 	        	$name = $tag[1][1];
			   	  	 	        	//Check if field type is text
			   	  	 	        	if(!empty($vardefs[$name]['type']) && $vardefs[$name]['type'] == 'text') {
			   	  	 	        	  $def = array();
			   	  	 	        	  $def['name'] = $name;	    	
			   	  	 	        	} else {
			   	  	 	        	  $def = $name;
			   	  	 	        	}
			   	  	 	        } else {
			   	  	 	        	//At this point we don't have a tag that matches so let's try to 
			   	  	 	        	//see if the PHP file attempted a direct variable assignment from $focus instance
			   	  	 	        	$def = $this->findAssignedVariableName($tag[1][1], $filePath);
			   	  	 	            //$def = $tag[1][1];
			   	  	 	        } //if-else
			   	  	 	    } //if
			   	  	 } //foreach

		   	  	 } //if-else
		   	  	 $metacolumns[] = $def;
	   	  	 }
	   	  } else {
	   	  	 if(isset($def)) {
	   	  	 	preg_match_all('/[\{](MOD[\.]|APP[\.])LBL_([^\}].*?)[\}]/si', $field, $matches, PREG_SET_ORDER);
	   	  	 	if($matches && count($matches[0]) == 3) {
		   	  	 	$label = $matches[0][2];

                    $reset = false;
		   	  	 	if(is_array($def) && !isset($def['label']) && $def['name'] != $label) {
		   	  	 	   $def2 = $def;
		   	  	 	   $def2['label'] = 'LBL_' . strtoupper($label);
		   	  	 	   $reset = true;	
		   	  	 	} else if(!is_array($def) && $def != $label) {
		   	  	 	   $temp = $def;
		   	  	 	   $def2 = array();
		   	  	 	   $def2['name'] = $temp;
		   	  	 	   $def2['label'] = 'LBL_' . strtoupper($label);
		   	  	 	   $reset = true;
		   	  	 	} //if-else
		   	  	 	
		   	  	 	// Reset the last entry in the Array to use the new label attributes
		   	  	 	if($reset) {
		   	  	 	   array_pop($metacolumns);
		   	  	 	   $metacolumns[] = $def2;
		   	  	 	} //if
	   	  	 	} //if
	   	  	 }
	   	  } //if-else
	   	  
	   } //foreach($tablecolumns as $tcols)

   	   $metarow[] = array_reverse($metacolumns);
   } //foreach($tablerows as $trow) 
   
   
   $id = $tableCount == 0 ? 'default' : $tableCount;
   $tableCount++;
   $panels[$id] = $metarow;
   
} //foreach($tables as $table)

$templateMeta = array();
   
if($merge && !empty($masterCopy) && file_exists($masterCopy)) {

   $panels = $this->mergePanels($panels, $moduleDir, $masterCopy);
   $templateMeta = $this->mergeTemplateMeta($templateMeta, $moduleDir, $masterCopy);
}

return $this->createFileContents($moduleDir, $panels, $templateMeta, $filePath);

}

}
?>

