<?php
if (! defined('sugarEntry') || ! sugarEntry)
    die('Not A Valid Entry Point');
/**
 * SugarCRM is a customer relationship management program developed by
 * SugarCRM, Inc. Copyright (C) 2004 - 2007 SugarCRM Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License version 3 as published by the
 * Free Software Foundation with the addition of the following permission added
 * to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED WORK
 * IN WHICH THE COPYRIGHT IS OWNED BY SUGARCRM, SUGARCRM DISCLAIMS THE WARRANTY
 * OF NON INFRINGEMENT OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, see http://www.gnu.org/licenses or write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301 USA.
 * 
 * You can contact SugarCRM, Inc. headquarters at 10050 North Wolfe Road,
 * SW2-130, Cupertino, CA 95014, USA. or at email address contact@sugarcrm.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU General Public License version 3,
 * these Appropriate Legal Notices must retain the display of the "Powered by
 * SugarCRM" logo. If the display of the logo is not reasonably feasible for
 * technical reasons, the Appropriate Legal Notices must display the words
 * "Powered by SugarCRM".
 */



require_once ('modules/ModuleBuilder/parsers/parser.modifylistview.php');

class ParserModifySubPanel extends ParserModifyListView
{
    
    var $listViewDefs = false;
    var $defaults = array();
    var $additional = array();
    var $available = array();
    var $columns = array('LBL_DEFAULT' => 'getDefaultFields', 'LBL_HIDDEN' => 'getAvailableFields');

    function init ($module_name, $child_module)
    {
        $GLOBALS['log']->debug("in ParserModifySubPanel: module_name={$module_name} child_module={$child_module}");
        $this->parent_module = $module_name;
        $this->child_module = $child_module;
        global $beanList, $beanFiles;
        $class = $beanList[$module_name];
        require_once ($beanFiles[$class]);
        require_once ('include/SubPanel/SubPanelDefinitions.php');
        $this->module = new $class();
        $spd = new SubPanelDefinitions($this->module);
        $spd->open_layout_defs(true, '', true);
        $original_panel = $spd->load_subpanel($child_module, true, true);
        $this->originalListViewDefs = $original_panel->get_list_fields();
        $spd->open_layout_defs(true, '', false);
        $this->panel = $spd->load_subpanel($child_module, true);
        $this->listViewDefs = $this->panel->get_list_fields();
        $this->language_module = $this->panel->template_instance->module_dir;
    }

    /**
     * returns the default fields for a listview
     */
    function getDefaultFields ()
    {
        $this->defaults = array();
        foreach ($this->listViewDefs as $key => $def)
        {
            if (! empty($def['usage']) && strcmp($def['usage'], 'query_only') == 0)
                continue;
            if (! empty($def['vname']))
                $def['label'] = $def['vname'];
            $this->defaults[$key] = $def;
        }
        return $this->defaults;
    }

    function getField ($fieldName)
    {
        foreach ($this->listViewDefs as $key => $def)
        {
            $key = strtolower($key);
            if ($key == $fieldName)
            {
                return $def;
            }
        }
        foreach ($this->originalListViewDefs as $key => $def)
        {
            $key = strtolower($key);
            if ($key == $fieldName)
            {
                return $def;
            }
        }
        foreach ($this->panel->template_instance->field_defs as $key => $def)
        {
            $key = strtolower($key);
            if ($key == $fieldName)
            {
                return $def;
            }
        }
        return array();
    }

    function handleSave ()
    {
        $GLOBALS['log']->debug("in ParserModifySubPanel->handleSave()");
        require_once ('include/SubPanel/SubPanel.php');
        $this->subpanel = new SubPanel($this->parent_module, 'fab4', $this->child_module, $this->panel);
        $newFields = array();
        foreach ($this->listViewDefs as $name => $field)
        {
            if (! isset($field['usage']) || $field['usage'] != 'query_only')
            {
                $existingFields[$name] = $field;
            
            } else
            {
                $newFields[$name] = $field;
            }
        }
        
        global $beanList, $beanFiles;
        $cMod = ! preg_match("/^[a-z]/", $this->child_module) ? $this->child_module : strtoupper($this->child_module[0]) . substr($this->child_module, 1);
        $class = $beanList[$cMod];
//      $GLOBALS['log']->debug("Class = $class");
        $mod = new $class();
        
        foreach ($_REQUEST['group_0'] as $field)
        {
            if (! empty($this->originalListViewDefs[$field]))
            {
                $newFields[$field] = $this->originalListViewDefs[$field];
            } 
            else
            {
                
                $vname = '';
                if (isset($this->panel->template_instance->field_defs[$field]))
                {
                    $vname = $this->panel->template_instance->field_defs[$field]['vname'];
                }
                
                if (isset($mod->field_name_map[$field]) && ($mod->field_name_map[$field]['type'] == 'bool' || (isset($mod->field_name_map[$field]['custom_type']) && $mod->field_name_map[$field]['custom_type'] == 'bool')))
                {
                    $newFields[$field] = array('name' => $field, 'vname' => $vname, 'widget_type' => 'checkbox');
                } else
                {
                    $newFields[$field] = array('name' => $field, 'vname' => $vname);
                }
            }
            
            if (isset($_REQUEST[strtolower($field) . 'width']))
            {
                $width = substr($_REQUEST[strtolower($field) . 'width'], 6, 3);
                if (strpos($width, "%") != false)
                {
                    $width = substr($width, 0, 2);
                }
                if ($width < 101 && $width > 0)
                {
                    $newFields[$field]['width'] = $width;
                }
            }
            else if (isset($this->listViewDefs[$field]['width']))
            {
                $newFields[$field]['width'] = $this->listViewDefs[$field]['width'];
            }
        }
        $this->subpanel->saveSubPanelDefOverride($this->panel, 'list_fields', $newFields);
    
    }

}
?>
